@@ -1,5 +1,5 @@
 /*
- * Copyright 2019-2020 OARC, Inc.
+ * Copyright 2019-2023 OARC, Inc.
  * Copyright 2017-2018 Akamai Technologies
  * Copyright 2006-2016 Nominum, Inc.
  * All rights reserved.
@@ -17,21 +17,20 @@
  * limitations under the License.
  */
 
+#include "config.h"
+
+#include "os.h"
+
+#include "log.h"
+#include "util.h"
+
 #include <errno.h>
 #include <signal.h>
 #include <inttypes.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
-
-#include <sys/select.h>
-
-#include <isc/result.h>
-#include <isc/types.h>
-
-#include "log.h"
-#include "os.h"
-#include "util.h"
+#include <poll.h>
 
 void perf_os_blocksignal(int sig, bool block)
 {
@@ -59,99 +58,94 @@ void perf_os_handlesignal(int sig, void (*handler)(int))
     }
 }
 
-isc_result_t
+perf_result_t
 perf_os_waituntilreadable(struct perf_net_socket* sock, int pipe_fd, int64_t timeout)
 {
-    return perf_os_waituntilanyreadable(sock, 1, pipe_fd, timeout);
+    struct perf_net_socket* socks[] = { sock };
+    return perf_os_waituntilanyreadable(socks, 1, pipe_fd, timeout);
 }
 
-isc_result_t
-perf_os_waituntilanyreadable(struct perf_net_socket* socks, unsigned int nfds, int pipe_fd,
+perf_result_t
+perf_os_waituntilanyreadable(struct perf_net_socket** socks, unsigned int nfds, int pipe_fd,
     int64_t timeout)
 {
-    fd_set         read_fds;
-    unsigned int   i;
-    int            maxfd;
-    struct timeval tv, *tvp;
-    int            n;
+    struct pollfd fds[nfds + 1];
+    size_t        i;
+    int           to, n;
 
-    FD_ZERO(&read_fds);
-    maxfd = 0;
     for (i = 0; i < nfds; i++) {
-        if (socks[i].have_more)
-            return (ISC_R_SUCCESS);
-        FD_SET(socks[i].fd, &read_fds);
-        if (socks[i].fd > maxfd)
-            maxfd = socks[i].fd;
+        if (perf_net_have_more(socks[i]))
+            return (PERF_R_SUCCESS);
+
+        fds[i].fd     = socks[i]->fd;
+        fds[i].events = POLLIN;
     }
-    FD_SET(pipe_fd, &read_fds);
-    if (pipe_fd > maxfd)
-        maxfd = pipe_fd;
+
+    fds[nfds].fd     = pipe_fd;
+    fds[nfds].events = POLLIN;
 
     if (timeout < 0) {
-        tvp = NULL;
+        to = -1;
     } else {
-        tv.tv_sec  = timeout / MILLION;
-        tv.tv_usec = timeout % MILLION;
-        tvp        = &tv;
+        to = timeout / 1000;
+        if (timeout && !to) {
+            to = 1;
+        }
     }
-    n = select(maxfd + 1, &read_fds, NULL, NULL, tvp);
+
+    n = poll(fds, nfds + 1, to);
     if (n < 0) {
         if (errno != EINTR) {
             char __s[256];
             perf_log_fatal("select(): %s", perf_strerror_r(errno, __s, sizeof(__s)));
         }
-        return (ISC_R_CANCELED);
+        return (PERF_R_CANCELED);
     } else if (n == 0) {
-        return (ISC_R_TIMEDOUT);
-    } else if (FD_ISSET(pipe_fd, &read_fds)) {
-        return (ISC_R_CANCELED);
+        return (PERF_R_TIMEDOUT);
+    } else if (fds[nfds].revents & POLLIN) {
+        return (PERF_R_CANCELED);
     } else {
-        return (ISC_R_SUCCESS);
+        return (PERF_R_SUCCESS);
     }
 }
 
-isc_result_t
-perf_os_waituntilanywritable(struct perf_net_socket* socks, unsigned int nfds, int pipe_fd,
+perf_result_t
+perf_os_waituntilanywritable(struct perf_net_socket** socks, unsigned int nfds, int pipe_fd,
     int64_t timeout)
 {
-    fd_set         write_fds, read_fds;
-    unsigned int   i;
-    int            maxfd;
-    struct timeval tv, *tvp;
-    int            n;
+    struct pollfd fds[nfds + 1];
+    size_t        i;
+    int           to, n;
 
-    FD_ZERO(&read_fds);
-    FD_ZERO(&write_fds);
-    maxfd = 0;
     for (i = 0; i < nfds; i++) {
-        FD_SET(socks[i].fd, &write_fds);
-        if (socks[i].fd > maxfd)
-            maxfd = socks[i].fd;
+        fds[i].fd     = socks[i]->fd;
+        fds[i].events = POLLOUT;
     }
-    FD_SET(pipe_fd, &read_fds);
-    if (pipe_fd > maxfd)
-        maxfd = pipe_fd;
+
+    fds[nfds].fd     = pipe_fd;
+    fds[nfds].events = POLLIN;
 
     if (timeout < 0) {
-        tvp = NULL;
+        to = -1;
     } else {
-        tv.tv_sec  = timeout / MILLION;
-        tv.tv_usec = timeout % MILLION;
-        tvp        = &tv;
+        to = timeout / 1000;
+        if (timeout && !to) {
+            to = 1;
+        }
     }
-    n = select(maxfd + 1, &read_fds, &write_fds, NULL, tvp);
+
+    n = poll(fds, nfds + 1, to);
     if (n < 0) {
         if (errno != EINTR) {
             char __s[256];
             perf_log_fatal("select(): %s", perf_strerror_r(errno, __s, sizeof(__s)));
         }
-        return (ISC_R_CANCELED);
+        return (PERF_R_CANCELED);
     } else if (n == 0) {
-        return (ISC_R_TIMEDOUT);
-    } else if (FD_ISSET(pipe_fd, &read_fds)) {
-        return (ISC_R_CANCELED);
+        return (PERF_R_TIMEDOUT);
+    } else if (fds[nfds].revents & POLLIN) {
+        return (PERF_R_CANCELED);
     } else {
-        return (ISC_R_SUCCESS);
+        return (PERF_R_SUCCESS);
     }
 }
