@@ -1,5 +1,5 @@
 /*
- * Copyright 2019-2020 OARC, Inc.
+ * Copyright 2019-2023 OARC, Inc.
  * Copyright 2017-2018 Akamai Technologies
  * Copyright 2006-2016 Nominum, Inc.
  * All rights reserved.
@@ -18,6 +18,7 @@
  */
 
 #include "net.h"
+#include "result.h"
 
 #ifndef PERF_OS_H
 #define PERF_OS_H 1
@@ -29,15 +30,15 @@ void perf_os_blocksignal(int sig, bool block);
 
 void perf_os_handlesignal(int sig, void (*handler)(int));
 
-isc_result_t
+perf_result_t
 perf_os_waituntilreadable(struct perf_net_socket* sock, int pipe_fd, int64_t timeout);
 
-isc_result_t
-perf_os_waituntilanyreadable(struct perf_net_socket* socks, unsigned int nfds, int pipe_fd,
+perf_result_t
+perf_os_waituntilanyreadable(struct perf_net_socket** socks, unsigned int nfds, int pipe_fd,
     int64_t timeout);
 
-isc_result_t
-perf_os_waituntilanywritable(struct perf_net_socket* socks, unsigned int nfds, int pipe_fd,
+perf_result_t
+perf_os_waituntilanywritable(struct perf_net_socket** socks, unsigned int nfds, int pipe_fd,
     int64_t timeout);
 
 #endif
