@@ -1,5 +1,5 @@
 /*
- * Copyright 2019-2021 OARC, Inc.
+ * Copyright 2019-2023 OARC, Inc.
  * Copyright 2017-2018 Akamai Technologies
  * Copyright 2006-2016 Nominum, Inc.
  * All rights reserved.
@@ -255,6 +255,8 @@ static void setup(int argc, char** argv)
     const char*  doh_method      = DEFAULT_DOH_METHOD;
     const char*  local_suppress  = 0;
 
+    size_t num_queries_per_conn = 0;
+
     sock_family     = AF_UNSPEC;
     server_port     = 0;
     local_port      = DEFAULT_LOCAL_PORT;
@@ -337,6 +339,8 @@ static void setup(int argc, char** argv)
         "the HTTP method to use for DNS-over-HTTPS: GET or POST", DEFAULT_DOH_METHOD, &doh_method);
     perf_long_opt_add("suppress", perf_opt_string, "message[,message,...]",
         "suppress messages/warnings, see dnsperf(1) man-page for list of message types", NULL, &local_suppress);
+    perf_long_opt_add("num-queries-per-conn", perf_opt_uint, "queries",
+        "Number of queries to send per connection", NULL, &num_queries_per_conn);
 
     perf_opt_parse(argc, argv);
 
@@ -397,7 +401,7 @@ static void setup(int argc, char** argv)
     perf_net_parselocal(server_addr.sa.sa.sa_family, local_name,
         local_port, &local_addr);
 
-    input = perf_datafile_open(filename);
+    input = perf_datafile_open(filename, input_format_text_query);
     if (reopen_datafile) {
         perf_datafile_setmaxruns(input, -1);
     }
@@ -428,6 +432,9 @@ static void setup(int argc, char** argv)
         if (!socks[i]) {
             perf_log_fatal("perf_net_opensocket(): no socket returned, out of memory?");
         }
+        if (num_queries_per_conn && socks[i]->num_queries_per_conn) {
+            socks[i]->num_queries_per_conn(socks[i], num_queries_per_conn, query_timeout);
+        }
     }
 }
 
@@ -628,7 +635,7 @@ do_one_line(perf_buffer_t* lines, perf_buffer_t* msg)
     }
 
     perf_buffer_clear(lines);
-    result = perf_datafile_next(input, lines, false);
+    result = perf_datafile_next(input, lines);
     if (result != PERF_R_SUCCESS)
         perf_log_fatal("ran out of query data");
     perf_buffer_usedregion(lines, &used);
@@ -738,7 +745,9 @@ try_process_response(unsigned int sockindex)
 
     size_t idx = qid * nsocks + sockindex;
     if (idx >= max_outstanding || queries[idx].list != &outstanding_list) {
-        perf_log_warning("received a response with an unexpected id: %u", qid);
+        if (!suppress.unexpected) {
+            perf_log_warning("received a response with an unexpected id: %u", qid);
+        }
         return;
     }
     q = &queries[idx];
